"""
exE-2.py

B(E)3M33UI - Artificial Intelligence course, FEE CTU in Prague
Model evaluation and diagnostics

Petr Posik, Jiri Spilka, CVUT, Praha 2017
"""

import numpy as np
from matplotlib import pyplot as plt

from sklearn.utils import shuffle
from sklearn import cross_validation
from sklearn.svm import SVC

from plotting import compute_learning_curve, plot_learning_curve, compute_validation_curve, plot_validation_curve, \
    plot_xy_classified_data, plot_2D_class_model


def main():

    X, y = generate_data(n1=400, n2=400)
    X, y = shuffle(X, y)

    plot_xy_classified_data(X[:, 0], X[:, 1], y)

    Xtr, Xtst, ytr, ytst = cross_validation.train_test_split(X, y, test_size=0.20)

    # Create a classifier
    model = SVC(kernel='rbf', gamma=.001)

    # FIXME Tasks 14-16 - Learning curves
    # Construct the array of training set sizes we are interested in.
    tr_sizes = np.arange(40, ytr.shape[0], 10)
    # raise NotImplementedError
    # <YOUR CODE HERE>

    train_errors, test_errors = compute_learning_curve(model, tr_sizes, Xtr, ytr, Xtst, ytst)
    plot_learning_curve(tr_sizes, train_errors, test_errors)

    plt.figure()
    model.fit(X, y)
    plot_xy_classified_data(X[:, 0], X[:, 1], y)
    plot_2D_class_model(model)
    plt.title('Model on the complete dataset: ')

    # FIXME Tasks 17-18 - Validation curve
    raise NotImplementedError
    # <YOUR CODE HERE>

    train_errors, test_errors = compute_validation_curve(model, param_name, param_range, Xtr, ytr, Xtst, ytst)
    plot_validation_curve(param_range, train_errors, test_errors)

    raise NotImplementedError


def generate_data(n1=500, n2=500, sigma=0.04):

    cov = [[sigma, 0], [0, sigma]]

    xn1 = np.random.multivariate_normal([+0.4, 0.7], cov, int(n1/2))
    xn2 = np.random.multivariate_normal([-0.3, 0.7], cov, int(n1/2))
    xn = np.vstack((xn1, xn2))

    xp1 = np.random.multivariate_normal([-0.7, 0.3], cov, int(n2/2))
    xp2 = np.random.multivariate_normal([+0.3, 0.3], cov, int(n2/2))
    xp = np.vstack((xp1, xp2))

    X = np.vstack((xn, xp))
    y = np.hstack((np.zeros((n1,), dtype=np.int8), np.ones((n2,), dtype=np.int8))).T

    return X, y


# catch all exceptions
try:
    plt.ion()  # turn interactive mode on so our plots stay open like in matlab
    main()
except BaseException as e:
    import traceback
    traceback.print_exc()  # mimic printing traceback from an exception
    plt.ioff()  # turn interactive mode off
    plt.show()  # show whatever we have to show, it will stay open because we turned interactive mode off

